@@ -1,3 +1,4 @@
+// Copyright 2018 Google LLC. All Rights Reserved.
 /*
   Copyright (C) 2005-2015 Steven L. Scott
 
@@ -16,55 +17,41 @@
   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 */
 
-#include <Models/LognormalModel.hpp>
-#include <distributions.hpp>
-#include <cpputil/math_utils.hpp>
+#include "Models/LognormalModel.hpp"
+#include "cpputil/math_utils.hpp"
+#include "distributions.hpp"
 
 namespace BOOM {
 
   LognormalModel::LognormalModel(double mu, double sigma)
-      : ParamPolicy(new UnivParams(mu),
-                    new UnivParams(sigma * sigma)),
-        DataPolicy(new GaussianSuf)
-  {
+      : ParamPolicy(new UnivParams(mu), new UnivParams(sigma * sigma)),
+        DataPolicy(new GaussianSuf) {
     if (sigma <= 0) {
       report_error("Standard deviation must be positive.");
     }
   }
 
-  LognormalModel::LognormalModel(Ptr<UnivParams> mu,
-                                 Ptr<UnivParams> sigsq)
-      : ParamPolicy(mu, sigsq),
-        DataPolicy(new GaussianSuf)
-  {
+  LognormalModel::LognormalModel(const Ptr<UnivParams> &mu,
+                                 const Ptr<UnivParams> &sigsq)
+      : ParamPolicy(mu, sigsq), DataPolicy(new GaussianSuf) {
     if (sigsq->value() <= 0) {
       report_error("Variance must be positive.");
     }
   }
 
-  LognormalModel * LognormalModel::clone() const {
+  LognormalModel *LognormalModel::clone() const {
     return new LognormalModel(*this);
   }
 
-  Ptr<UnivParams> LognormalModel::Mu_prm() {
-    return prm1();
-  }
+  Ptr<UnivParams> LognormalModel::Mu_prm() { return prm1(); }
 
-  Ptr<UnivParams> LognormalModel::Sigsq_prm() {
-    return prm2();
-  }
+  Ptr<UnivParams> LognormalModel::Sigsq_prm() { return prm2(); }
 
-  double LognormalModel::mu() const {
-    return prm1_ref().value();
-  }
+  double LognormalModel::mu() const { return prm1_ref().value(); }
 
-  double LognormalModel::sigsq() const {
-    return prm2_ref().value();
-  }
+  double LognormalModel::sigsq() const { return prm2_ref().value(); }
 
-  void LognormalModel::set_mu(double mu) {
-    prm1_ref().set(mu);
-  }
+  void LognormalModel::set_mu(double mu) { prm1_ref().set(mu); }
 
   void LognormalModel::set_sigsq(double sigsq) {
     if (sigsq <= 0) {
@@ -73,15 +60,14 @@ namespace BOOM {
     prm2_ref().set(sigsq);
   }
 
-  double LognormalModel::mean() const {
-    return exp(mu() + 0.5 * sigsq());
-  }
+  double LognormalModel::mean() const { return exp(mu() + 0.5 * sigsq()); }
 
   double LognormalModel::variance() const {
-    return (exp(sigsq()) - 1) * square(mean());
+    return expm1(sigsq()) * square(mean());
   }
 
-  double LognormalModel::Logp(double x, double &d1, double &d2, uint nderiv) const {
+  double LognormalModel::Logp(double x, double &d1, double &d2,
+                              uint nderiv) const {
     if (nderiv > 0) {
       double logx = log(x);
       double residual = logx - mu();
