@@ -1,3 +1,4 @@
+// Copyright 2018 Google LLC. All Rights Reserved.
 /*
   Copyright (C) 2005-2010 Steven L. Scott
 
@@ -16,36 +17,34 @@
   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 */
 
-#include <numopt/ScalarNewtonMax.hpp>
-#include <cpputil/report_error.hpp>
+#include "numopt/ScalarNewtonMax.hpp"
+#include "cpputil/report_error.hpp"
 
-namespace BOOM{
-  double scalar_newton_max(const d2ScalarTargetFun &f,
-                           double &x,
-                           double &g,
+namespace BOOM {
+  double scalar_newton_max(const d2ScalarTargetFun &f, double &x, double &g,
                            double &h) {
-    double y = f(x,g,h);
+    double y = f(x, g, h);
     double oldy = y;
-    double eps = 1e-5;  // TODO(stevescott):  remove magic numbers
+    double eps = 1e-5;  // TODO:  remove magic numbers
     double dy = eps + 1;
-    while(dy > eps){
-      double step = g/h;
-      if(h > 0) step = -1*step;
+    while (dy > eps) {
+      double step = g / h;
+      if (h > 0) step = -1 * step;
       double oldx = x;
       x -= step;
-      y = f(x,g,h);
+      y = f(x, g, h);
       dy = y - oldy;
-      while(dy < 0){
-        step/=2.0;
+      while (dy < 0) {
+        step /= 2.0;
         x = oldx - step;
-        if(fabs(step) < eps){
+        if (fabs(step) < eps) {
           report_error("too small a step size in scalar_newton_max");
         }
-        y = f(x,g,h);
+        y = f(x, g, h);
         dy = y - oldy;
       }
       oldy = y;
     }
     return y;
   }
-}
+}  // namespace BOOM
